@@ -1,6 +1,6 @@
 /* client_keyexchange.c
  *
- * $Id: client_keyexchange.c,v 1.52 2000/09/19 18:13:02 nisse Exp $ */
+ * $Id: client_keyexchange.c,v 1.53 2000/10/31 21:38:29 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -117,7 +117,7 @@ do_handle_dh_reply(struct packet_handler *c,
   
   /* Key exchange successful! */
 
-  connection->dispatch[SSH_MSG_KEXDH_REPLY] = connection->fail;
+  connection->dispatch[SSH_MSG_KEXDH_REPLY] = &connection_fail_handler;
 
   keyexchange_finish(connection, closure->algorithms,
 		     closure->dh.method->H,
@@ -204,7 +204,7 @@ do_srp_client_proof_handler(struct packet_handler *s,
 				     packet);
   lsh_string_free(packet);
 
-  connection->dispatch[SSH_MSG_KEXSRP_PROOF] = connection->fail;
+  connection->dispatch[SSH_MSG_KEXSRP_PROOF] = &connection_fail_handler;
   
   if (res)
     {
@@ -251,7 +251,7 @@ do_handle_srp_reply(struct packet_handler *s,
   
   mpz_t x;
   
-  connection->dispatch[SSH_MSG_KEXSRP_REPLY] = connection->fail;
+  connection->dispatch[SSH_MSG_KEXSRP_REPLY] = &connection_fail_handler;
     
   if (!salt)
     {
