@@ -1,6 +1,6 @@
 /* connection_commands.h
  *
- * $Id: connection_commands.h,v 1.14 2000/10/06 23:52:46 nisse Exp $
+ * $Id: connection_commands.h,v 1.15 2000/10/31 21:43:57 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -67,7 +67,8 @@ make_handshake_info(UINT32 flags,
 extern struct collect_info_1 handshake_command;
 #define CONNECTION_HANDSHAKE (&handshake_command.super.super.super)
 
-struct lsh_callback *make_connection_close_handler(struct ssh_connection *c);
+struct lsh_callback *
+make_connection_close_handler(struct ssh_connection *c);
 
 extern struct command_simple connection_remember;
 #define CONNECTION_REMEMBER (&connection_remember.super.super)
