@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: dh_exchange.c,v 1.8 2000/09/03 17:09:16 nisse Exp $ */
+ * $Id: dh_exchange.c,v 1.10 2000/09/19 18:14:40 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -65,12 +65,6 @@ init_dh_instance(struct dh_method *m,
   HASH_UPDATE(self->hash, s->length, s->data);
 
   lsh_string_free(s);  
-
-  /* We don't need the kexinit strings anymore. */
-  lsh_string_free(c->literal_kexinits[CONNECTION_CLIENT]);
-  lsh_string_free(c->literal_kexinits[CONNECTION_SERVER]);
-  c->literal_kexinits[CONNECTION_CLIENT] = NULL;
-  c->literal_kexinits[CONNECTION_SERVER] = NULL;
 }
 
 struct dh_method *
@@ -177,6 +171,7 @@ dh_make_server_secret(struct dh_instance *self)
 struct lsh_string *
 dh_make_server_msg(struct dh_instance *self,
 		   struct lsh_string *server_key,
+		   int hostkey_algorithm,
 		   struct signer *s)
 {
   dh_hash_update(self, ssh_format("%S", server_key), 1);
@@ -185,7 +180,7 @@ dh_make_server_msg(struct dh_instance *self,
   return ssh_format("%c%S%n%fS",
 		    SSH_MSG_KEXDH_REPLY,
 		    server_key,
-		    self->f, SIGN(s,
+		    self->f, SIGN(s, hostkey_algorithm,
 				  self->exchange_hash->length,
 				  self->exchange_hash->data));
 }
