@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: io.h,v 1.64 2000/10/09 23:33:16 nisse Exp $ */
+ * $Id: io.h,v 1.66 2000/11/14 17:41:24 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -185,7 +185,23 @@ void init_consuming_read(struct io_consuming_read *self,
        ; The port number here is always in host byte order
        (port . UINT32))) */
 
-/* Returned by listen. And also by connect, so this is an improper name. */
+/* Used for listening and connecting to local sockets.
+ * Both strings have to be NUL-terminated. */
+
+/* GABA:
+   (class
+     (name local_info)
+     (vars
+       (directory string)
+       (name string)))
+*/
+
+struct local_info *
+make_local_info(struct lsh_string *directory,
+		struct lsh_string *name);
+     
+/* Returned by listen. And also by connect, so this is an improper name.
+ * Functions related to AF_UNIX sockets leave the peer field as NULL. */
 /* GABA:
    (class
      (name listen_value)
@@ -279,6 +295,9 @@ write_raw(int fd, UINT32 length, const UINT8 *data);
 const struct exception *
 write_raw_with_poll(int fd, UINT32 length, const UINT8 *data);
 
+const struct exception *
+read_raw(int fd, UINT32 length, UINT8 *data);
+
 void io_set_nonblocking(int fd);
 void io_set_close_on_exec(int fd);
 void io_init_fd(int fd);
@@ -308,15 +327,13 @@ io_listen(struct io_backend *b,
 
 struct lsh_fd *
 io_listen_local(struct io_backend *b,
-		struct lsh_string *directory,
-		struct lsh_string *name,
+		struct local_info *info,
 		struct io_callback *callback,
 		struct exception_handler *e);
 
 struct lsh_fd *
 io_connect_local(struct io_backend *b,
-		 struct lsh_string *directory,
-		 struct lsh_string *name,
+		 struct local_info *info,
 		 struct command_continuation *c,
 		 struct exception_handler *e);
 
@@ -325,6 +342,12 @@ make_listen_callback(struct io_backend *backend,
 		     struct command_continuation *c,
 		     struct exception_handler *e);
 
+#if 0
+struct io_callback *
+make_listen_callback_no_peer(struct io_backend *backend,
+			     struct command_continuation *c);
+#endif
+
 struct lsh_fd *io_read_write(struct lsh_fd *fd,
 			     struct io_callback *read,
 			     UINT32 block_size,
@@ -359,12 +382,14 @@ io_read_file(struct io_backend *backend,
 	     const char *fname, 
 	     struct exception_handler *e);
 
+#if 0
 struct lsh_fd *
 io_read_user_file(struct io_backend *backend,
 		  const char *fname,
 		  uid_t uid, int secret,
 		  const struct exception **x,
 		  struct exception_handler *e);
+#endif
 
 int
 lsh_make_pipe(int *fds);
