@@ -2,7 +2,7 @@
  *
  * main proxy program.
  *
- * $Id: lsh_proxy.c,v 1.14 2000/10/17 21:54:02 nisse Exp $ */
+ * $Id: lsh_proxy.c,v 1.17 2000/11/14 19:59:25 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -95,9 +95,10 @@ struct command_simple proxy_destination;
 #include <unistd.h>
 #endif
 
+#if 0
 /* Block size for stdout and stderr buffers */
 #define BLOCK_SIZE 32768
-
+#endif
 
 /* Option parsing */
 
@@ -127,6 +128,7 @@ const char *argp_program_bug_address = BUG_ADDRESS;
      (super algorithms_options)
      (vars
        (backend object io_backend)
+       (random object randomness_with_poll)
        (signature_algorithms object alist)
        (style . sexp_argp_state)
        (interface . "char *")
@@ -146,7 +148,7 @@ const char *argp_program_bug_address = BUG_ADDRESS;
 
 static struct lsh_proxy_options *
 make_lsh_proxy_options(struct io_backend *backend, 
-		       struct randomness *random, 
+		       struct randomness_with_poll *random, 
 		       struct alist *algorithms)
 {
   NEW(lsh_proxy_options, self);
@@ -154,9 +156,10 @@ make_lsh_proxy_options(struct io_backend *backend,
   init_algorithms_options(&self->super, algorithms);
   self->signature_algorithms
     = make_alist(1,
-		 ATOM_DSA, make_dsa_algorithm(random), -1);
+		 ATOM_DSA, make_dsa_algorithm(&random->super), -1);
 
   self->backend = backend;
+  self->random = random;
   self->style = SEXP_TRANSPORT;
   self->interface = NULL;
 
@@ -183,14 +186,14 @@ make_lsh_proxy_options(struct io_backend *backend,
 }
 
 /* Port to listen on */
-COMMAND_SIMPLE(options2local)
+DEFINE_COMMAND_SIMPLE(options2local, a)
 {
   CAST(lsh_proxy_options, options, a);
   return &options->local->super;
 }
 
 /* alist of signature algorithms */
-COMMAND_SIMPLE(options2signature_algorithms)
+DEFINE_COMMAND_SIMPLE(options2signature_algorithms, a)
 {
   CAST(lsh_proxy_options, options, a);
   return &options->signature_algorithms->super;
@@ -315,6 +318,9 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 		   self->port);
       if (self->use_pid_file < 0)
 	self->use_pid_file = self->daemonic;
+
+      /* Start background poll */
+      RANDOM_POLL_BACKGROUND(self->random->poller);
       
       break;
       
@@ -482,7 +488,7 @@ make_proxy_destination_command(struct lsh_proxy_options *options)
   return &self->super;
 }
 
-COMMAND_SIMPLE(proxy_destination)
+DEFINE_COMMAND_SIMPLE(proxy_destination, a)
 {
   CAST(lsh_proxy_options, options, a);
   return &make_proxy_destination_command(options)->super;
@@ -572,6 +578,7 @@ COMMAND_SIMPLE(proxy_destination)
 #include "proxy_alf.h"
 #endif
 
+
 int main(int argc, char **argv)
 {
   struct lsh_proxy_options *options;
@@ -580,12 +587,12 @@ int main(int argc, char **argv)
   
   struct reap *reaper;
   
-  struct randomness *r;
+  struct randomness_with_poll *r;
   struct alist *algorithms_server, *algorithms_client;
   struct alist *signature_algorithms;
   
   struct make_kexinit *make_kexinit;
-
+  struct exception_handler *handler;
   
   NEW(io_backend, backend);
   init_backend(backend);
@@ -602,13 +609,18 @@ int main(int argc, char **argv)
   /* FIXME: Choose character set depending on the locale */
   set_local_charset(CHARSET_LATIN1);
 
-  r = make_reasonably_random();
+  handler = make_report_exception_handler
+    (make_report_exception_info(EXC_IO, EXC_IO, "lsh_proxy: "),
+     &default_exception_handler,
+     HANDLER_CONTEXT);
+  reaper = make_reaper();  
+  r = make_default_random(reaper, handler);
 
   algorithms_server = all_symmetric_algorithms();
   /* FIXME: copy algorithms_server */
   algorithms_client = all_symmetric_algorithms();
   
-  signature_algorithms = all_signature_algorithms(r);
+  signature_algorithms = all_signature_algorithms(&r->super);
   
   options = make_lsh_proxy_options(backend, r, algorithms_server);
   
@@ -668,8 +680,6 @@ int main(int argc, char **argv)
    * We should also extract the host-key algorithms for which we have keys,
    * instead of hardcoding ssh-dss below. */
  
-  reaper = make_reaper();
-
 #if 0
   lookup_keys = make_alist(1, 
 			   ATOM_SSH_DSS, make_fake_host_db(),
@@ -677,12 +687,14 @@ int main(int argc, char **argv)
 #endif
 
   ALIST_SET(algorithms_server, 
-	    ATOM_DIFFIE_HELLMAN_GROUP1_SHA1, make_dh_server(make_dh1(r)));
+	    ATOM_DIFFIE_HELLMAN_GROUP1_SHA1,
+	    make_dh_server(make_dh1(&r->super)));
   ALIST_SET(algorithms_client, 
-	    ATOM_DIFFIE_HELLMAN_GROUP1_SHA1, make_dh_client(make_dh1(r)));
+	    ATOM_DIFFIE_HELLMAN_GROUP1_SHA1,
+	    make_dh_client(make_dh1(&r->super)));
   
   make_kexinit
-    = make_simple_kexinit(r,
+    = make_simple_kexinit(&r->super,
 			  make_int_list(1, ATOM_DIFFIE_HELLMAN_GROUP1_SHA1,
 					-1),
 			  options->super.hostkey_algorithms,
@@ -796,7 +808,7 @@ int main(int argc, char **argv)
 						       "lsh_proxy_client - a free ssh",
 						       "proxy client",
 						       SSH_MAX_PACKET,
-						       r,
+						       &r->super,
 						       algorithms_client,
 						       NULL),
 						      make_kexinit),
@@ -805,7 +817,7 @@ int main(int argc, char **argv)
 						       "lsh_proxy_server - a free ssh",
 						       "proxy server",
 						       SSH_MAX_PACKET,
-						       r,
+						       &r->super,
 						       algorithms_server,
 						       NULL),
 						      make_kexinit));
@@ -816,10 +828,7 @@ int main(int argc, char **argv)
 
       COMMAND_CALL(server_listen, options,
                    &discard_continuation,
-                   make_report_exception_handler
-                   (make_report_exception_info(EXC_IO, EXC_IO, "lsh_proxy: "),
-                    &default_exception_handler,
-                    HANDLER_CONTEXT));
+		   handler);
     }
   }
   
