@@ -2,7 +2,7 @@
  *
  * main server program.
  *
- * $Id: lshd.c,v 1.99 2000/10/15 20:19:57 nisse Exp $ */
+ * $Id: lshd.c,v 1.102 2000/11/14 20:02:14 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -87,9 +87,10 @@ struct command_simple options2signature_algorithms;
 #include <unistd.h>
 #endif
 
+#if 0
 /* Block size for stdout and stderr buffers */
 #define BLOCK_SIZE 32768
-
+#endif
 
 /* Option parsing */
 
@@ -139,8 +140,11 @@ const char *argp_program_bug_address = BUG_ADDRESS;
      (super algorithms_options)
      (vars
        (backend object io_backend)
+       (e object exception_handler)
+       
        (reaper object reap)
-       (random object randomness)
+       (random object randomness_with_poll)
+       
        (signature_algorithms object alist)
        (style . sexp_argp_state)
        (interface . "char *")
@@ -173,6 +177,29 @@ const char *argp_program_bug_address = BUG_ADDRESS;
        (use_pid_file . int)))
 */
 
+static void
+do_exc_lshd_handler(struct exception_handler *s,
+		    const struct exception *e)
+{
+  switch(e->type)
+    {
+    case EXC_SEXP_SYNTAX:
+    case EXC_SPKI_TYPE:
+    case EXC_RANDOMNESS_LOW_ENTROPY:
+      werror("lshd: %z\n", e->msg);
+      exit(EXIT_FAILURE);
+    default:
+      EXCEPTION_RAISE(s->parent, e);
+    }
+}
+
+static struct exception_handler *
+make_lshd_exception_handler(struct exception_handler *parent,
+			    const char *context)
+{
+  return make_exception_handler(do_exc_lshd_handler, parent, context);
+}
+
 static struct lshd_options *
 make_lshd_options(struct io_backend *backend)
 {
@@ -181,11 +208,12 @@ make_lshd_options(struct io_backend *backend)
   init_algorithms_options(&self->super, all_symmetric_algorithms());
 
   self->backend = backend;
+  self->e = make_lshd_exception_handler(&default_exception_handler,
+					HANDLER_CONTEXT);
   self->reaper = make_reaper();
-  self->random = make_reasonably_random();
+  self->random = make_default_random(self->reaper, self->e);
 
-  /* FIXME: We don't support rsa yet in the rest of the code! */
-  self->signature_algorithms = all_signature_algorithms(self->random);
+  self->signature_algorithms = all_signature_algorithms(&self->random->super);
   self->style = SEXP_TRANSPORT;
   self->interface = NULL;
 
@@ -224,14 +252,14 @@ make_lshd_options(struct io_backend *backend)
 }
 
 /* Port to listen on */
-COMMAND_SIMPLE(options2local)
+DEFINE_COMMAND_SIMPLE(options2local, a)
 {
   CAST(lshd_options, options, a);
   return &options->local->super;
 }
 
 /* alist of signature algorithms */
-COMMAND_SIMPLE(options2signature_algorithms)
+DEFINE_COMMAND_SIMPLE(options2signature_algorithms, a)
 {
   CAST(lshd_options, options, a);
   return &options->signature_algorithms->super;
@@ -373,7 +401,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 		LIST(self->kex_algorithms)[i++] = ATOM_DIFFIE_HELLMAN_GROUP1_SHA1;
 		ALIST_SET(self->super.algorithms,
 			  ATOM_DIFFIE_HELLMAN_GROUP1_SHA1,
-			  make_dh_server(make_dh1(self->random)));
+			  make_dh_server(make_dh1(&self->random->super)));
 	      }
 #if WITH_SRP	    
 	    if (self->with_srp_keyexchange)
@@ -382,7 +410,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 		LIST(self->kex_algorithms)[i++] = ATOM_SRP_RING1_SHA1_LOCAL;
 		ALIST_SET(self->super.algorithms,
 			  ATOM_SRP_RING1_SHA1_LOCAL,
-			  make_srp_server(make_srp1(self->random), db));
+			  make_srp_server(make_srp1(&self->random->super), db));
 	      }
 #endif /* WITH_SRP */
 	  }
@@ -434,6 +462,9 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	  }
 	else
 	  argp_error(state, "All user authentication methods disabled.");
+
+	/* Start background poll */
+	RANDOM_POLL_BACKGROUND(self->random->poller);
 	
 	break;
       }
@@ -596,27 +627,6 @@ main_argp =
 	      (connection_require_userauth connection)))))))
 */
 
-static void
-do_lshd_default_handler(struct exception_handler *s,
-			const struct exception *e)
-{
-  switch(e->type)
-    {
-    case EXC_SEXP_SYNTAX:
-    case EXC_SPKI_TYPE:
-      werror("lshd: %z\n", e->msg);
-      exit(EXIT_FAILURE);
-    default:
-      EXCEPTION_RAISE(s->parent, e);
-    }
-}
-
-static struct exception_handler *
-make_lshd_exception_handler(struct exception_handler *parent,
-			    const char *context)
-{
-  return make_exception_handler(do_lshd_default_handler, parent, context);
-}
 
 int main(int argc, char **argv)
 {
@@ -672,7 +682,7 @@ int main(int argc, char **argv)
       werror("lshd seems to be running already.\n");
       return EXIT_FAILURE;
     }
-    
+
   {
     /* Commands to be invoked on the connection */
     struct object_list *connection_hooks;
@@ -722,11 +732,11 @@ int main(int argc, char **argv)
 					"lsh - a free ssh",
 					NULL,
 					SSH_MAX_PACKET,
-					options->random,
+					&options->random->super,
 					options->super.algorithms,
 					options->sshd1),
 		    make_simple_kexinit
-		    (options->random,
+		    (&options->random->super,
 		     options->kex_algorithms,
 		     options->super.hostkey_algorithms,
 		     options->super.crypto_algorithms,
@@ -747,8 +757,7 @@ int main(int argc, char **argv)
 		   &discard_continuation,
 		   make_report_exception_handler
 		   (make_report_exception_info(EXC_IO, EXC_IO, "lshd: "),
-		    make_lshd_exception_handler(&default_exception_handler,
-						HANDLER_CONTEXT),
+		    options->e,
 		    HANDLER_CONTEXT));
     }
   }
