@@ -1,6 +1,6 @@
 /* server.c
  *
- * $Id: server.c,v 1.63 2000/06/27 11:36:41 nisse Exp $ */
+ * $Id: server.c,v 1.64 2000/10/31 22:33:27 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -91,7 +91,7 @@ do_service_request(struct packet_handler *c,
 	    {
 	      /* Don't accept any further service requests */
 	      connection->dispatch[SSH_MSG_SERVICE_REQUEST]
-		= connection->fail;
+		= &connection_fail_handler;
 
 	      /* Start service */
 #if DATAFELLOWS_WORKAROUNDS
