@@ -2,7 +2,7 @@
  *
  * Reads a sexp in given form from, and writes it in given form.
  *
- * $Id: sexp-conv.c,v 1.2 2000/10/03 21:45:55 nisse Exp $ */
+ * $Id: sexp-conv.c,v 1.4 2000/10/17 23:46:26 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -31,7 +31,7 @@
 #include "lsh.h"
 #include "lsh_argp.h"
 #include "sexp_commands.h"
-#include "spki.h"
+#include "spki_commands.h"
 #include "version.h"
 #include "werror.h"
 #include "xalloc.h"
@@ -367,7 +367,9 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 
 static const struct argp
 main_argp =
-{ main_options, main_argp_parser, NULL,
+{ main_options, main_argp_parser,
+  "Conversion: sexp-conv [options] <INPUT-SEXP >OUTPUT\n"
+  "Fingerprinting: sexp-conv --raw-hash [ --hash=ALGORITHM ] <PUBLIC-KEY",
   "Reads an s-expression on stdin, and outputs the same "
   "s-expression on stdout, possibly using a different "
   "encoding. By default, output uses the advanced encoding. ",
