@@ -2,7 +2,7 @@
  *
  * Create an SRP verifier
  *
- * $Id: srp-gen.c,v 1.3 2000/06/01 18:06:32 nisse Exp $ */
+ * $Id: srp-gen.c,v 1.4 2000/11/14 20:08:36 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -99,7 +99,7 @@ make_srp_gen_options(struct io_backend *backend,
   self->passwd = NULL;
 
   /* We use this only for generating the salt. */
-  self->r = make_reasonably_random();
+  self->r = make_bad_random();
 
   return self;
 }
